Option Explicit

' "Smart" front end to Beyond Compare
'
' v2.0    (2014/04/09) simpler, cleaner rewrite
' v2.1    (2014/04/09) add user configurable path to BCompare.exe via Preferences dialog
'         (thanks to wowbagger for the suggestion)
'         use fsu.resolve consistently to fully resolve paths
' v2.2    (2014/04/10) add proper versioning and default icon
' v2.2 LD (2014/04/29) Modified version by Leo Davidson:
'                      - Requires Directory Opus 11.3.1 or above.
'                      - Ensures left and right in BeyondCompare match the left and right in Opus (even if the right is the source).
'                      - Allows comparing folders with archives (since BC supports this), with a new config option to disable it.
'                      - If one item is selected in the source and nothing (or the wrong kind of thing, or too many things) in the dest,
'                        it now ignores the destination selection(s) and compares the selected source item on both sides. For example, if
'                        C:\ and D:\ are the source/dest folders, and File.txt is selected in the source, C:\File.txt and D:\File.txt are compared.
'                        "Wrong kind of thing" means a file on one side and a folder on the other. (Where archives are considered folders.)
'                      - Fix for when the Desktop is compared to another folder. (Compares /desktopdir instead of /desktop virtual folder.)
'                      - Gets default exe path from the registry if the script config has no path or has a path that doesn't exist.
'                        Failing that, it uses the standard Beyond Compare install path, under the 32-bit Program Files folder.
'                        Default config path is now blank, and should only be set if the above methods need to be overridden.
'                      - Shows an error message if Beyond Compare exe isn't found.
'                      - If the path found/configured is for BCompare.exe, it will swap it for BComp.exe (if it exists).
'                        Using BComp.exe is preferable as it will open new tabs in an existing BC instance. (Could make this an option.)
'                      - Won't do anything if nothing is to be compared.
'                        (If people want the button to be able to run BC on its own, it could be a config option.
'                         For me it'd just mean having to close BC if I clicked in error, so I added the check.)
'                      - Won't do anything if either thing to be compared does not exist.
'                        (This is mainly for the new case where you can select a file on one side to compare the same thing on both sides.)
'                      - New small icon matching the 22x22 size of Opus's other small toolbar icons, with 1 pixel padding.
'                        (Made by resizing the 32x32 icon to 20x20, then adding 1 pixel of padding on all sides, to match the standard icons.)
'                      - Removed debug stuff I didn't need, changed command name, and tided up the code.
' v2.3 AB (2014/05/02) Original code with Leo's improvements plus..
'                      - More flexible and rigorous handling of user supplied path/filespec
'                      - AB's added command is named $BC (vs Leo's BeyondCompare)
'                      - Uses AB's preferred logic for comparing selected files/folders.
'                         1. Compare files if exactly two files are selected in source.
'                         2. Compare folders if exactly two folders are selected in source.
'                         3. Compare relevant items if exactly one file or one folder is selected in each side of a DUAL.
'                         4. If none of the above and DUAL then compare sides (folders).
'                         5. If none of the above just open Beyond Compare main menu.

Function OnInit(ByRef initData) ' OnInit is called by DOpus to initialize the script
   
	initData.name = "$BC (Smart Beyond Compare)"
	initData.version= "2.3 AB"
	initData.desc = "Run Beyond Compare on selected files and folders."
	initData.copyright = "(c) 2014 AussieBoykie"
	initData.default_enable = true
	initData.min_version = "1.3.1"

	initData.config.BCPath = ""
	initData.config.ArchivesAsFolders = True

	dim cmd : set cmd = initData.AddCommand()
	cmd.name = "$BC" ' This is the name of the command being added
	cmd.method = "Main" ' This is the routine to execute when the BeyondCompare command is invoked 
	cmd.desc = initData.desc
	cmd.label = "Compare"
	cmd.icon = "BeyondCompare:BeyondCompare"
	cmd.template = ""

End Function

Function Main(ByRef scriptCmdData)

	Dim cdf : Set cdf = scriptCmdData.func
	Dim fsu : Set fsu = DOpus.FSUtil

	Dim tempPath

	' Prevent deselection. This this at the start so it's done even if we exit early.   
	cdf.command.deselect = false

	Dim getTypeMode : getTypeMode = ""
	If (Not Script.config.ArchivesAsFolders) Then
		getTypeMode = "a"
	End If

	' ********************************
	' *** Get path to BCompare.exe ***
	' ********************************
   
   ' The user might supply a path to BCompare.exe - e.g. C:\Utilities\BC
   ' or a full filespec - e.g. C:\Utilities\BC\BCompare.exe
   ' If a path is supplied we also need to eliminate unwanted leading or trailing spaces
	Dim BCPath : BCPath = Trim(Script.config.BCPath)
   If Not (Trim(BCPath) = "") Then
      If 0 = InStr(1, LCase(BCPath), ".exe", 1 ) Then BCPath = Trim(BCPath) & "\BCompare.exe"
   End If

	If (BCPath = "") Or (Not fsu.Exists(BCPath)) Then
		Const HKEY_CLASSES_ROOT   = &H80000000
		Const HKEY_CURRENT_USER   = &H80000001
		Const HKEY_LOCAL_MACHINE  = &H80000002
		Const HKEY_USERS          = &H80000003
		Dim reg : Set reg = GetObject("winmgmts:{impersonationLevel=impersonate}!\\.\root\default:StdRegProv")
		Dim strValue
		reg.GetStringValue HKEY_CURRENT_USER, "Software\Scooter Software\Beyond Compare", "ExePath", strValue
		If (Not IsNull(strValue)) Then
	        	BCPath = strValue
		Else
			BCPath = fsu.resolve("/programfilesx86\Beyond Compare 3\BCompare.exe")
		End If
	End If

	If (BCPath = "") Or (Not fsu.Exists(Trim(BCPath))) Then
		cdf.Dlg.Request "BCompare.exe could not be found automatically. You can configure the script with an explicit path if needed.", "OK", "Beyond Compare script"
		Exit Function
	End If

	' ******************************************************
	' *** Change BCompare.exe to BComp.exe, if available ***
	' ******************************************************
   
	Set tempPath = fsu.NewPath(BCPath)
	If tempPath.filepart = "BCompare.exe" Then
		tempPath.Parent
		tempPath.Add "BComp.exe"
		If fsu.Exists(tempPath) Then
			BCPath = tempPath
		End If
	End If
	Set tempPath = Nothing
   
	' ****************************
	' *** Get paths to compare ***
	' ****************************

	Dim leftPath  : Set leftPath = Nothing
	Dim rightPath : Set rightPath = Nothing

	' We compare types using fsu.GetType, rather than using selected_dirs and selected_files,
	' because we want the feature of fsu.GetType where it (by default) considers archives
	' as folders. This is because Beyond Compare can do the same and it allows people to
	' e.g. compare archive backups against the folders they were made from.
	' The user can turn this mode off, which affects the getTypeMode variable.

	If (cdf.sourcetab.selected_files.count = 2) Then
		' Exactly two files are selected in SOURCE (Don't care if folders are also selected)
		Set leftPath = fsu.NewPath(cdf.sourcetab.selected_files(0))
		Set rightPath = fsu.NewPath(cdf.sourcetab.selected_files(1))
	ElseIf (cdf.sourcetab.selected_dirs.count = 2) Then
		' Exactly two folders are selected in SOURCE (Don't care if files are also selected)
		Set leftPath = fsu.NewPath(cdf.sourcetab.selected(0))
		Set rightPath = fsu.NewPath(cdf.sourcetab.selected(1))
	ElseIf cdf.desttab <> 0 Then
		If cdf.sourcetab.selected.count = 1 And cdf.desttab.selected.count = 1 Then
			' Separate if-statement because VBScript doesn't have short-circuit logic (SIGH!)
			' and it's not safe to access selected(0) until we know the counts are non-zero.
			If fsu.GetType(cdf.sourcetab.selected(0), getTypeMode) = fsu.GetType(cdf.desttab.selected(0), getTypeMode) Then
				' A single item of the same type selected in both SOURCE and DEST
				Set leftPath = fsu.NewPath(cdf.sourcetab.selected(0))
				Set rightPath = fsu.NewPath(cdf.desttab.selected(0))
			Else
				' SOURCE and DEST folders will be compared
				Set leftPath = fsu.NewPath(cdf.sourcetab.path)
				Set rightPath = fsu.NewPath(cdf.desttab.path)
			End If
      Else
         ' SOURCE and DEST folders will be compared
         Set leftPath = fsu.NewPath(cdf.sourcetab.path)
         Set rightPath = fsu.NewPath(cdf.desttab.path)
		End If
	End If

	' **************************
	' *** Build command line ***
	' **************************

	' Basic BC command with no arguments
	Dim cmdline : cmdline = """" & BCPath & """"

	If (leftPath Is Nothing) Or (rightPath Is Nothing) Then
		' If nothing is to be compared, just run BC on its own.
		' (Could make this a config option for those who would prefer the button to do nothing.)
		' Exit Function   
	Else
		If cdf.command.IsSet("SOURCE=Right") Then
			' The paths are the wrong way around, so swap them.
			Set tempPath = leftPath
			Set leftPath = rightPath
			Set rightPath = tempPath
			Set tempPath = Nothing
		End If

		' Use DisplayName "er" instead of Resolve, as we need to not only resolve the paths
		' but also convert virtual folder paths, like Desktop, into normal paths.
		leftPath = fsu.DisplayName(leftPath, "er")
		rightPath = fsu.DisplayName(rightPath, "er")

		' Kludge: At the time of writing, a bug in Opus means fsu.Exists returns false for zip archives.
		'         Instead, use fsu.GetType and test against "invalid".
		If (fsu.GetType(leftPath)) = "invalid" Or (fsu.GetType(rightPath) = "invalid") Then
			Exit Function
		End If

		' Test that the two things being compared are of the same file/dir type.
		If fsu.GetType(leftPath, getTypeMode) <> fsu.GetType(rightPath, getTypeMode) Then
			Exit Function
		End If

		cmdline = cmdline & " " & """" & leftPath & """" & " " & """" & rightPath & """"
	End If
   
	' The REPLACE function in the following statement appears to be needed
	' to prevent the possibility of (e.g.) %1, %2, etc. being misinterpreted

	cmdline = Replace(cmdline, "%", "%%") ' Execute Beyond Compare with appropriate args 

	' ***********************
	' *** Run the command ***
	' ***********************

	cdf.command.runcommand cmdline
   
End Function
